C	Main run file for QLD model using HOMPACK90 with DENSE ("FULL")Jacobian
C	- reads model parameters and starting value of variables 
C	               from input file (hom_start.dat)
C	- sets up and runs the homotopy
C	Developed by Steve Kryukov, with assistance from Paul Grieco and Guy Arie
C	Based on EZ_hompack by K. Schmedders and HOMPACK90's test2.f
C     HOMPACK90 is developped by L. Whatson et. al (1997)

CCC	When switching to NEW Model, change the parameter read statements in 
C	SUBROUTINE hom_Initialize1Run(fname, fid, traceID, N)
C	and possibly "read vector variables" in Main program

CCC
CCC Module Declarations
CCC
	MODULE SWITCH
C	ROWSET  IS USED TO INITIALIZE SPARSE MATRIX DATA STRUCTURES ONLY
C	ONCE, AFTER THEY ARE ALLOCATED.
	LOGICAL::  ROWSET
	END MODULE SWITCH

C	profit function - global variable since COMMON requires constant size
	MODULE mProfit
	USE REAL_PRECISION
      REAL (KIND=R8), DIMENSION(:), ALLOCATABLE:: Profit ! Profit, allocatable
!	INTEGER ndx(18,18)
	END MODULE mProfit

CCC	****************************************************
CCC	Main program
CCC
	PROGRAM maindriver

	USE SWITCH
	USE REAL_PRECISION, ONLY : R8
!	USE HOMPACK90_GLOBAL, ONLY: QRSPARSE, ROWPOS, COLPOS ! only for test
	USE mProfit ! profit function
C	uncomment one of the lines for the algorithm that you use
!	USE HOMPACK90, ONLY : FIXPDF  ! ODE-based
!	USE HOMPACK90, ONLY : FIXPNF  ! Normal Flow
	USE HOMPACK90, ONLY : FIXPQF  ! Augmented Jac
	
	IMPLICIT NONE

! model dimensions, constants specified to declare vector
! must be changed for new model


C	variables for file input/output - do not change
	integer ios, fid ! input-output status
	parameter (fid=13) ! file handle	
	character* 100 startFileName
	character* 10 batchId

CCC     hompack90 variables - do not change
	REAL (KIND=R8), DIMENSION(:), ALLOCATABLE::A, Y, X, H
	REAL (KIND=R8):: ANSAE,ANSRE,ARCAE,ARCRE,
     &    ARCLEN, SSPAR(8)
	INTEGER:: IFLAG,II,LENQR,MODE,NFE,N,NP1,TIMENEW(8),
     &   TIMEOLD(8),TRACE,NDIMA
	parameter (TRACE=0) ! output file handle, 0 for none
C       service variables - do not change
	integer i,j, argmaxH
	real*8 maxH, temp(14), lambda

CCC	function interfaces - letting Fortran know which functions will be called
      INTERFACE
  	  SUBROUTINE hom_Initialize1Run(fname, fid,traceID,  N)
		USE REAL_PRECISION, ONLY : R8
		USE mProfit
		CHARACTER* (*), INTENT(IN):: fname 
		INTEGER, INTENT(IN)::fid, traceID
		INTEGER, INTENT(OUT)::N
	  END SUBROUTINE

	  SUBROUTINE hom_InitializeStepfiles(startFileName)
		character* (*),	INTENT(OUT)::startFileName
	  END SUBROUTINE

	  SUBROUTINE hom_TestRun(H, A, X, N)
			USE REAL_PRECISION, ONLY : R8
			REAL (KIND=R8), INTENT(OUT) :: H(:)
			REAL (KIND=R8), INTENT(OUT) :: A(:),X(:)
			INTEGER ,		INTENT(IN) :: N
	  END SUBROUTINE

	END INTERFACE
CCC
CCC Section 1. READ / SETUP PARAMETERS AND VARIABLES
CCC

C	determine working directory	
	CALL hom_InitializeStepfiles(startFileName)	

C	open input file, read parameters return file handle fid
	CALL hom_Initialize1Run(trim(startFileName),fid,TRACE,N)

C	Allocate space for vector variables
C	space is de-allocated at end of maindriver routine
	ALLOCATE(A(N))
	ALLOCATE(Y(N+1))
	ALLOCATE(X(N))
	ALLOCATE(H(N))
	ALLOCATE(Profit(N/2))

C	read vector variables and close input file
	read(fid) (X(i),i=1,N)
	read(fid) (Profit(i),i=1,N/2)
	close(fid)

CCC
CCC Section 2. TEST COMPONENTS
CCC

	CALL hom_TestRun(H,A,X,N)

CCC
CCC Section 3. HOMOTOPY
CCC
!	DEFINE ARGUMENTS FOR CALL TO HOMPACK PROCEDURE.
!     only thing you can change is precision (ARCRE - ANSAE)
	MODE=2
	NP1=N+1
	NDIMA = N
	Y(2:NP1)=X(1:N)
	Y(1)=0
	ARCRE=1.0D-10 ! path (arc) precision, relative and absolute
	ARCAE=1.0D-10
	ANSRE=1.0D-12 ! answer precision, relative and absolute
	ANSAE=1.0D-12
	SSPAR=0.0
!	SSPAR(5) = 2 ! maximum step size for Normal Flow & Augmented Jac methosd
	IFLAG=-2 ! -2 for path following, do not change
	IF(IFLAG .EQ. -2) A=Y(1:N)
	ROWSET = .FALSE.

CCC	**** run the homotopy ****
C	(un)comment the lines to switch algorithms;
C	 do not forget to (un)comment "USE HOMPACK90, ... " line above

C	ODE-based
!      CALL FIXPDF(N,Y,IFLAG,ARCRE,ANSRE,TRACE,A,NDIMA,NFE,
!     &      ARCLEN)

C	Normal-flow
      CALL FIXPNF(N,Y,IFLAG,ARCRE,ARCAE,ANSRE,ANSAE,TRACE,A,
     &   SSPAR,NFE,ARCLEN)

C	Augmented-Jaconbian
!      CALL FIXPQF(N,Y,IFLAG,ARCRE,ARCAE,ANSRE,ANSAE,TRACE,A,
!     &     SSPAR,NFE,ARCLEN)

	write(*,'(A,I2)'), 'homotopy (fixP?F) done, flag=', IFLAG
	IF (TRACE .GT. 0) THEN
	  write(TRACE,'(A,I2)'), 'homotopy (fixP?S) done, flag=', IFLAG
	ENDIF

	DEALLOCATE(A)
	DEALLOCATE(Y)
	DEALLOCATE(X)
	DEALLOCATE(H)
	IF (ALLOCATED(Profit)) DEALLOCATE(Profit)

	END PROGRAM maindriver
CCC	end of main program

CCC	********************************************************
CCC	SUBROUTINE hom_Initialize1Run(fname, fid, traceID, N)
CCC    - declares and reads model parameters from input file
CCC	      returns file handle fid to calling routine so it can read vector variables

	SUBROUTINE hom_Initialize1Run(fname, fid, traceID, N)
    
     
	USE REAL_PRECISION, ONLY : R8
	USE mProfit
	CHARACTER* (*), INTENT(IN):: fname 
	INTEGER, INTENT(IN)::fid, traceID
	INTEGER , INTENT(OUT) :: N
	
C	variables for file input/output 
	integer ios,  i ! input-output status
	

! model parameters - must be changed for new model
	integer L
	real*8 beta
	real*8 a_beg, a_end
	real*8 d_beg, d_end
	COMMON/ModelParams/beta,a_beg,a_end,d_beg,d_end,L


! offsets in X and H vectors - must be changed for new model
! not necessary, but make indexing and re-ordering columns/rows easier
	integer offV, offZ ! X vector: value V and GZ Zeta
	integer offBell, offFOC ! H vector: Bellman and GZ FOC
	COMMON/OFFSETS/offV,offZ,offBell,offFOC

	real*8 temp(14)


CCC	Open input file
C     #ifdef is a preprocesser directive that switches from FORM='BINARY' 
C               to ACCESS='STREAM' to accomodate GFORTRAN (GCC)
C	User might need to activate preprocessor - read documentation on your compiler
C	When using GFortran, add "-dGFORTRAN" to command line option 
C              This requires GCC 4.3 or better
#ifdef GFORTRAN
	open( UNIT=fid, FILE=fname, STATUS="OLD",
     & ACTION = "READ", IOSTAT = ios, ACCESS = "STREAM")
#else
	open( UNIT=fid, FILE=fname, STATUS="OLD",
     & ACTION = "READ", IOSTAT = ios, FORM = "BINARY")
#endif
	if (ios>0) then
		print *, "ios=", float(ios),"name=", fname
		stop "Fortran: cannot find or open hom_start.dat"
	else
		print *, "reading starting data"
	endif


CCC	read parameters - change for new model
	read(fid) (temp(i),i=1,6)
C	reminder on order of parameters: copy of Matlab code writing them
C	c1=fwrite(fid, [L B a a_end d d_end], 'float64');
      L=INT(temp(1))
      beta = temp(2)
	a_beg = temp(3)
	a_end = temp(4)
	d_beg = temp(5)
	d_end = temp(6)

	
CCC	compute N and offsets in X and H vectors
	N=L*L*2
	offV = 0
	offZ = L*L
	offBell = 0
	offFOC = L*L

	print *, "   Parameters and X-vector loaded:"
	write(*,'(A,F6.3,A,F7.3,F7.3,A,F7.3,F7.3,A)'),'beta=',beta,'
     +      alfa=[',a_beg, a_end, '] delta=[',d_beg, d_end,']'


CCC	open text output file, comment out if not needed
	IF (traceID .GT. 0) THEN
		open( UNIT=traceID, FILE="hom_out.txt", STATUS="REPLACE",
     &		ACTION = "WRITE", IOSTAT = ios, FORM = "FORMATTED")
		if (ios>0) stop "cannot open output file"

	
		 write(traceID,'(A,F6.3,A,F7.3,F7.3,A,F7.3,F7.3,A)'),
     &	 'beta=',beta,'
     +      alfa=[',a_beg, a_end, '] delta=[',d_beg, d_end,']'
	endif


	END
CCC	end of SUBROUTINE hom_Initialize1Run(fname, fid, traceID, N)
	

C	***************************************
CCC	SUBROUTINE hom_TestRun(H, A, X, N)
CCC	   simple test runs of system of equations and Jacobian
CCC	comment out lines to skip test
	SUBROUTINE hom_TestRun(H, A, X, N)

	USE REAL_PRECISION, ONLY : R8
	
      INTERFACE
	  SUBROUTINE RHO (A,LAMBDA,X,H)
          USE REAL_PRECISION, ONLY : R8
		REAL (KIND=R8), INTENT(IN) :: A(:),X(:)
		REAL (KIND=R8), INTENT(IN OUT) :: LAMBDA
		REAL (KIND=R8), INTENT(OUT) :: H(:)
	  END SUBROUTINE RHO

      END INTERFACE

	REAL (KIND=R8), INTENT(OUT) :: H(:)
	REAL (KIND=R8), INTENT(IN) :: A(:),X(:)
	INTEGER ,		INTENT(IN) :: N
	
	REAL (KIND=R8) LAMBDA, maxH
	INTEGER i, argmaxH

CCC	test run of homotopy function (system of equations)
	H(1:N)=0.0
	lambda=0.0
	print *, 'testing system of equations (RHO function)'
	call RHO(A,lambda,X,H)

	maxH=0.0
	argmaxH=0
	do 120 i=1,N
	  if (dabs(H(i)).gt.maxH) then
     		maxH=dabs(H(i))
		argmaxH=i
	  endif
 120  continue

	write(*, '(A,E8.3,A,I4)'),'  RHO done, max(abs) = ', maxH,
     &                           ' ,element # ', argmaxH

	END
CCC	end of SUBROUTINE hom_TestRun(H, A, X, N)



CCC	plugs: subroutines are required for compilation,
C	but never called at runtime
      SUBROUTINE F(X,V)

	print *, 'F called'
      END SUBROUTINE F

      SUBROUTINE FJAC(X,V,K)
	print *, 'FJAC was called'
      RETURN
      END SUBROUTINE FJAC

      SUBROUTINE RHOA(A,LAMBDA,X)
	print *, 'RHOA was called'
      RETURN
      END SUBROUTINE RHOA

      SUBROUTINE FJACS(X)
	print *, 'FJACS was called'
      RETURN
      END SUBROUTINE FJACS

	SUBROUTINE RHOJS(A,LAMBDA,X)
      RETURN
      END SUBROUTINE RHOJS

CCC	Uncomment these if you do not have rhojs.f or rho.f files
C	homotopy will not run w/o them, but at least code will compile.

!	SUBROUTINE RHOJAC(A,LAMBDA,X,V,K)
!	print *, 'RHOJAC was called'
!	RETURN
!	END SUBROUTINE RHOJAC

!	SUBROUTINE RHO(A,LAMBDA,X,H)
!      RETURN
!      END SUBROUTINE RHO
C
