%%% read homotopy output, compute herfs and other things

% clear all
% close all
Path2Data='HomXpt\'; % path to homotopy files
% directory should contain input file hom_start.dat 
% and numbered output files hom_*.dat

% script automatically generates:
% hom_ad(:,2)=[alpha delta] for every step
% hom_ls(:,2)=[lambda s] for every step

% computationally intensive tasks
% can be done every N steps,
% or turned of by setting =0;
doHerfs=100; % Herfindahl computation
    % produces hom_Herfs(:,1:N+1)= [step Herf([T1 T2 ... TN])]
    % to get matching alpha&delta, use hom_ad(hom_Herfs(:),:)
    transT = [10 100 1000]; % T's where Herfs are computed (T1 .. TN)
	st = [0 0]; % T=0 state for transient distributions
    
doPMcheck=0; % PM check:  run a single PM iteration.
    % produces hom_PM(:,3) = [step dis(invt) dis(V)]
            %only for steps where check is done
    % this is a precision check, 
    % small distance means we are near equilibrium

    
if doHerfs
    hom_Herfs=zeros(0,1+length(transT)); % [step Herf] 
	%%% parameters of period game, price and profit functions    
    %%% comment out if kept in memory or loaded along with homotopy
% 	wstar = 12; % state at which utility function flattens out
% 	M = 5;    % market size
% 	c = 5;     % MC of production
%     % technical inits, do not touch
%     p0 = [10, 10]; % guess at solution of pricing FOC
%     maxiter = 1; epsilon = 1e-12;
%     w=[1:1:L]';nu = w;
%     for i=wstar+1:L % utility function
%         nu(i) = wstar + log(2 - exp(-(i-(wstar))));
%     end   
	%%% end of: parameters of period game, price and profit functions    
    
    run QLDsetup % compute profit, just in case
end

    
%%% read model parameters from starting file
fid = fopen('hom_start.dat','r'); % open file for write
if fid==-1
    error('Starting file not found or failed to open.')
end
tmp=fread(fid, 6, 'float64');
L=tmp(1);
homX=fread(fid,2*L*L,'float64'); % just to move read point to profit
homPrf=fread(fid,L*L,'float64');
fclose(fid);

B=tmp(2);
a_beg=tmp(3);
a_end=tmp(4);
d_beg=tmp(5);
d_end=tmp(6);

if max(abs(abs( profit(:,:,1) - reshape(homPrf,L,L) ))) > 1e-3
    disp('warning: wrong profit function. Did {wstar, M, c, nu(:)} change?');
end
profit(:,:,1)=reshape(homPrf,L,L); 

%%% initialize the loop 
offV=0; offZ=L*L; % x-vector offsets
step=1; % step number
done=0; % finish indicator
s0=0; % path length for previous step
% pre-define lists of homotopy stats (not really necessary)
hom_invt=zeros(0,2); % average invt and # of zero investment states
hom_ad=zeros(0,2); % a (alpha) and (d) delta
hom_ls=zeros(0,2); % lambda and path length s
% inits for PM check
if doPMcheck % i.e. doPMcheck>0
	epsilon = 1e-17; % stopping threshold
	maxiter = 1; % maximum # of iterations in value function iteration algorithm
    hom_PM=zeros(0,3); % step, dis(invt), dis(V)
end
if doHerfs
    hom_Herfs=zeros(0,2); % [step Herf] 
	% parameters of period game, price and profit functions    
    % should match hompack-starting code:
    % either keep in memory or save&load 
	wstar = 12; % state at which utility function flattens out
	M = 5;    % market size
	c = 5;     % MC of production
    % technical inits, do not touch
    p0 = [10, 10]; % guess at solution of pricing FOC
    maxiter = 1; epsilon = 1e-12;
    w=[1:1:L]';nu = w;
    for i=wstar+1:L % utility function
        nu(i) = wstar + log(2 - exp(-(i-(wstar))));
    end   
    run QLDsetup % compute profit
end

close all;

%%% main loop
while ~done
    
    filename=sprintf('%shom_%05d.dat',Path2Data,step);
    % check if file we ran out of files
    if ~exist(filename,'file') % no more files
        done=1;
continue
	end
    
    % read data from file
	fid = fopen(filename,'r'); % open file for write
	if fid==-1
        error([filename ' failed to open'])
	end
	homX=fread(fid,2*L*L,'float64');
    hom_ls(step,1)=fread(fid,1,'float64');
    s=fread(fid,1,'float64');
    fclose(fid);
    
    % process data into our format
    hom_ls(step,2)=s-s0;
    s0=s;
    
    hom_ad(step,1) = a_beg + hom_ls(step,1)*(a_end-a_beg);
    hom_ad(step,2) = d_beg + hom_ls(step,1)*(d_end-d_beg);
    a=hom_ad(step,1);
    d=hom_ad(step,2);
    
	v=reshape(homX(offV+1:offV+L*L),L,L); % v(:) is a single long vector, in "natural" order
	
	for j=1:L
	for i=1:L
		invt(i,j)=max(0,-homX(offZ+(j-1)*L+i))^2; 
        % this might be possible to do as matrix operation (use .^ for elementwise power)
	end 
	end
    
    % do some summary statistics
    hom_invt(step,1)=sum(sum(invt))/(L*L); % average investment
    hom_invt(step,2)=length(find( invt(:)<1e-8 )); % # of zero-investment states
    % 1e-8 is the precision of this check.
    
	if doPMcheck & mod(step,doPMcheck)==0
        a=hom_ad(step,1);
        d=hom_ad(step,2);
        Qldvfi
        hom_PM=[hom_PM; step disinvt(1) disv(1)];
    end
    
    if doHerfs & (mod(step,doHerfs)==0 | step==1)
        transfn2D=[];
        Qldvfi %
        
		QLD_TransHerfs
        hom_Herfs=[hom_Herfs; step HerfT];
%          run QLDlimdis
%          run QLDherf
%          hom_Herfs=[hom_Herfs; step expherf];
    end
    % disinvt(n), disv(n)
    
    step=step+1;
end % while ~done (main loop)
step=step-1;

% report results
disp(sprintf('%d steps, a in [%.3f,%.3f], d in [%.3f,%.3f]', step, ... 
       min(a,a_end), max(a,a_end), min(d,d_end), max(d,d_end)))
if doPMcheck
    disp(sprintf('max PM distances: invt=%.2e V=%.2e', max(hom_PM(:,2:3)) ))
end

%%% plot results

% determine which param changes along the path - alpha or delta
d_ad=max(hom_ad)-min(hom_ad);
if d_ad(1)>1e-2
    parStr='\alpha';
    parInd=1;    
else
    parStr='\delta';
    parInd=2;    
end

%%% basic homotopy stats
figure('Name','homotopy','NumberTitle','off');

ha=subplot(4,1,1); % 1st subplot in 4x1 panel set
hl=plot([1:step]',hom_ls(:,1)); % plot lambda
set(hl,'Linewidth',1.5);
ax=axis; ax(1:4)=[1 step 0 1]; axis(ax); % set axis limits
ylabel('\lambda');
title('Basic statistics')

ha=subplot(4,1,2); % 2nd subplot in 4x1 panel set
hl=plot([1:step]',hom_ls(:,2)); 
set(hl,'Linewidth',1.5);
ax=axis; ax(1:2)=[1 step]; axis(ax); % set axis limits
ylabel('Step ds');

ha=subplot(4,1,3); % 3d subplot 
hl=plot([1:step]',hom_ad(:,1)); 
set(hl,'Linewidth',1.5);
ax=axis; ax(1:4)=[1 step min(a_beg,a_end)-0.001 max(a_beg,a_end)+0.001]; axis(ax); % set axis limits
ylabel('\alpha');

ha=subplot(4,1,4); % 3d subplot 
hl=plot([1:step]',hom_ad(:,2)); 
set(hl,'Linewidth',1.5);
ax=axis; ax(1:4)=[1 step min(d_beg,d_end)-0.001 max(d_beg,d_end)+0.001]; axis(ax); % set axis limits
xlabel('step #');
ylabel('\delta');

%%% investment figure
figure('Name','Invt','NumberTitle','off');

ha=subplot(2,1,1);
hl=plot(hom_ad(:,parInd), hom_invt(:,1),'Linewidth',1.5); 
    ax=axis; 
    ax(1:2)=[min(hom_ad(:,parInd)) max(hom_ad(:,parInd))]; 
    axis(ax); % set axis limits
% xlabel('homotopy parameter \lambda');
ylabel('Average investment')
title('Investment statistics');

ha=subplot(2,1,2);
hl=plot(hom_ad(:,parInd),hom_invt(:,2),'Linewidth',1.5); 
    ax=axis; 
    ax(1:2)=[min(hom_ad(:,parInd)) max(hom_ad(:,parInd))]; 
    axis(ax); % set axis limits
xlabel(parStr);
ylabel('# states with invt=0');

% Herf plot
if doHerfs
    figure('Name','Herfs','NumberTitle','off');
    hl=plot(hom_ad(hom_Herfs(:,1),parInd),hom_Herfs(:,2:end));
        ax=axis; 
        ax(1:2)=[min(hom_ad(:,parInd)) max(hom_ad(:,parInd))]; 
        ax(3:4)=[0.5 1.0];
        axis(ax); % set axis limits
    legend('T=10','T=100','T=1000');
    xlabel(parStr); 
    ylabel('Herf'); 
    title('Transient Herfs');
end


