      SUBROUTINE QR_STRU(N,LENQR)
C	defines srarsity structure of Jacobian, in compacted row format
C	this implementation: Quality ladder model w/o entry or exit 

C	Nx(N+1) jacobian of rho(x,lambda)
C		columns 1:N = dRho/dx, column N+1 = dRho/dlambda

C	rows correspond to equations of homotopy functions, 
C	columns - to components of X vector
C	column N+1 is derivative w.r.t. homotopy paramter LAMBDA, and should be all nonzeros

C  QRSPARSE - non-zero elements of Jacobian matrix, in row order
C	we do not touch QRSPARSE here, it is filled by RHOJS
C	
C  LENQR is the length of QRSPARSE, used to allocate memory 
C  ROWPOS (length N+1) contains the indices of where each row begins within QRSPARSE
C	ROWPOS(N+1) = LENQR+1 by convention (so last row is like the rest)
C  COLPOS (of length LENQR) contains the column indices of
C    the corresponding elements in QRSPARSE.  

C	takes arguments defined in HOMPACK90_GLOBAL
C	N is number of equations (& variables)
C	returns in LENQR the length of QRSPARSE

CCC Instructions on editing this file for a new model
C  1. Copy-paste from Main.f or rho.f offsets and dimension parameters
C  2. Input AveZeros - 2*Number of policy functions + Number of states that you can transit to
C		you can make it lower if some rows have less nonzeros than others
C		you get runtime error "array bounds exceeded" if it's too low
C  3. Edit or comment out condition on "wrong N supplied to QR_STRU" 
C  4. Arrange loops so they go through equations in same order as in H vector (homotopy function)
C	If you use matlab's natural ordering, outer loop should be on J (column index), inner - I
C	You need a set of loops for every equation group (Bellman, FOC, exit/entry prob, etc.), 
C			and loops should go in same order as equation groups.
C	Remember to change line labels when copying loop code
C  5. Within each loop, set RR to row/equation number (can use formulas or do RR=RR+1)
C  6. Again within each loop, call RECNZ for each nonzero with column (=X-index) in 2nd argument. 
C	  RECNZ then enters it into COLPOS, and prepares for next element
C	  RECNZ also checks if there already was a nonzero in given column,
C		so can call it on own and rival's policy functions in symmetric states 
C  7. Make sure you put an nonzero on diagonal
C			
C


C	sparse structure
      USE HOMPACK90_GLOBAL, ONLY: QRSPARSE, ROWPOS, COLPOS
C	inputs
	INTEGER, INTENT(IN):: N
	INTEGER, INTENT(IN OUT):: LENQR

C	offsets in X and H vectors
	integer offV, offZ ! X vector: value V and GZ Zeta
	integer offBell, offFOC ! H vector: bellman and GZ FOC
	COMMON/OFFSETS/offV,offZ,offBell,offFOC

C	model parameters, only needed for L (gotta do entire block
	integer L
	real*8 beta
	real*8 a_beg, a_end
	real*8 d_beg, d_end
	COMMON/ModelParams/beta,a_beg,a_end,d_beg,d_end,L

C	indices & MISC
	INTEGER I,J, I0,I1, J1,J0, I2,J2 ! 2-dimensional state space
	INTEGER QPOS, RR ! position in sparse arrays
	INTEGER NP1 ! N+1

C	average number of zeros per row. start with max (nonzeros of Bellman), then decrease 
	INTEGER, PARAMETER:: AveZeros=15
	
	IF (N/=L*L*2) STOP "wrong N supplied to QR_STRU"
	NP1=N+1
	
C	ALLOCATE ARRAYS
      IF (ALLOCATED(QRSPARSE)) DEALLOCATE(QRSPARSE)
      IF (ALLOCATED(ROWPOS)) DEALLOCATE(ROWPOS)
      IF (ALLOCATED(COLPOS)) DEALLOCATE(COLPOS)
	LENQR = N*AveZeros
      ALLOCATE(QRSPARSE(LENQR))
      ALLOCATE(ROWPOS(N+2))
      ALLOCATE(COLPOS(LENQR))
	QRSPARSE=0.0
	ROWPOS=0
	COLPOS=0

	QPOS = 1

C	**** first set of equations/rows - Bellman 
C	you must go in same order as equations in rho.f
	DO 100 J=1,L
	DO 100 I=1,L
C	Edit this line to give proper row number 
C	= equation number in rho.f , = index used with H(...)
		RR = offBell + I+(J-1)*L
C	do not change this line, it intialized the row
		ROWPOS(RR) = QPOS
C	edit everything till end of loop, call RECNZ for every nonzero in row
C		change 2nd argument to column number ( = index in X-vector that you used in Rho.f)

C		policy		
		CALL RECNZ(QPOS,offZ+ I+(J-1)*L,LENQR,RR) ! own p, competitor active
		CALL RECNZ(QPOS,offZ+ J+(I-1)*L,LENQR,RR) ! competitor's p
		
C		Value for communicating states
		DO 120 J2=MAX(1,J-1),MIN(J+1,L)
		DO 120 I2=MAX(1,I-1),MIN(I+1,L)
C			 exclude (I-1,J+1) or (I+1,J-1) b/c we cannot transit there with common shock
		  IF ((I2-I)*(J2-J)/=-1) THEN 
			CALL RECNZ(QPOS, offV+ I2+(J2-1)*L, LENQR,RR)  
		  ENDIF
 120		CONTINUE

C homotopy parameter (lambda), last column - do not remove
		CALL RECNZ(QPOS,NP1,LENQR,RR) 
C diagonal element, just in case it got overlooked above
C	if it wasn't, RECNZ got uniqueness check
		CALL RECNZ(QPOS,RR,LENQR,RR) 

 100	CONTINUE ! I,J



!	 **** 2nd set of rows/equations - FOC
	DO 200 J=1,L
	DO 200 I=1,L
		RR = offFOC + I+(J-1)*L
		ROWPOS(RR) = QPOS

C		policy		
		CALL RECNZ(QPOS,offZ+ I+(J-1)*L,LENQR,RR) ! own p, competitor active
		CALL RECNZ(QPOS,offZ+ J+(I-1)*L,LENQR,RR) ! competitor's p
		
C		Value for communicating states
		DO 220 J2=MAX(1,J-1),MIN(J+1,L)
		DO 220 I2=MAX(1,I-1),MIN(I+1,L)
C			 exclude (I-1,J+1) or (I+1,J-1) b/c we cannot transit there with common shock
		  IF ((I2-I)*(J2-J)/=-1) THEN 
			CALL RECNZ(QPOS, offV+ I2+(J2-1)*L, LENQR,RR)  
		  ENDIF
 220		CONTINUE

C homotopy parameter (lambda), last column - do not change
		CALL RECNZ(QPOS,NP1,LENQR,RR) 
C diagonal element, just in case it got overlooked above
C	if it wasn't, RECNZ got uniqueness check
		CALL RECNZ(QPOS,RR,LENQR,RR) 

 200	CONTINUE ! I,J

CCC		
CCC	DO NOT EDIT ANYTHING BELOW THIS LINE
CCC
	ROWPOS(NP1) = QPOS
	LENQR=QPOS-1	
	WRITE(*, '(A,I6,A)'), ' sparse structure created, '
     &          ,QPOS-1, ' non-zeros'

C	SORT ROW INDECES - BUBLE (it's slow, but we only do it once)
	DO 400 I=1,N
		DO 440 J1=ROWPOS(I),(ROWPOS(I+1)-2)
		DO 440 J2=(J1+1),(ROWPOS(I+1)-1)
			IF ( COLPOS(J1) > COLPOS(J2)  ) THEN
				I0=COLPOS(J1)
				COLPOS(J1)=COLPOS(J2)
				COLPOS(J2)=I0
			END IF
 440		CONTINUE
 400	CONTINUE
	PRINT *, '   Jacobian structure sorted'

	END SUBROUTINE QR_STRU

C	*****************************************************************

	SUBROUTINE RECNZ(QPOS,K,LENQR,RR)
C	record a non-zero position in sparse structure, 
C	checking for max elements & unique entries (e.g. in case of symmetry)
      USE HOMPACK90_GLOBAL, ONLY: COLPOS, ROWPOS
	INTEGER, INTENT(IN):: K,LENQR,RR
	INTEGER, INTENT(IN OUT):: QPOS
	INTEGER I
!	LOGICAL OK  

	IF (QPOS>LENQR) STOP "more nonzeros in QR_STRU than MaxZeros allow"

C	check if K is already in
!	OK=.TRUE.
	DO 10 I=ROWPOS(RR),QPOS-1
		IF (COLPOS(I)==K) RETURN
 10	CONTINUE

!	IF (OK) THEN
		COLPOS(QPOS)=K
		QPOS=QPOS+1	
!	ENDIF
	RETURN
	END SUBROUTINE RECNZ
