% QLD_StartHom -- runs the homotopy:
% 	  -  loads saved equilibrium that will serve a starting point of homotopy,
% 	  -  writes the equilibrium and parameter values into binary file (hom_start.dat),
% 	  -  calls the executable compiled from Fortran90 file,
% 	  -  initiates output processing by running QLD_ReadHom.m and hom_steps.m.

%==================================================================================================================
%%% parameter values are set below
L = 18;    % # of states
% period game parameters
wstar = 12; % state at which utility function flattens out
M = 5;    % market size
c = 5;     % MC of production
% dynamic parameters
a = 15.0;     % effectiveness of investment (alpha)
d = 0.7;   % industry-wide depreciation (delta)
B = 0.925; % discount factor (beta)

% ending values of homotopy (beginning values = current)
a_end=5;
% due to reasons explained in the paper, 
% it is preferable to start with high $a$ and go down
d_end=d;
% we usually have either d_end=d; or a_end=a;
% this is only bc we are not interested in runs on two parameters
% homotopy will run just fine

%%% load saved equilibrium file
% note that "/" in filenames works fine on Windows Matlab 
fileX = sprintf('equilibria/L=%d,M=%d,c=%d,a=%0.2g,d=%0.2g,,B=%0.3g,wstar=%g.mat',L,M,c,a,d,B,wstar); 
if exist(fileX, 'file')
    load(fileX,'v','invt')
else
    error('Saved equilibrium not found, compute &save it with QLDmaster.m')
end
fileY = sprintf('PeriodProfit/L=%d,M=%d,c=%d,wstar=%g.mat',L,M,c,wstar); 
if exist(fileY, 'file')
    load(fileY,'profit');
else
    error('Saved period profits not found, compute & save them with QLDmaster.m')
end

%%% create vectors that will go into binary file
homX=repmat(NaN,2*L*L,1); % X vector - V, zeta
% homPrf=repmat(NaN,L*L,1); % profit vector
offV=0;
offZ=L*L;

homX(offV+1:offV+L*L)=v(:); % v(:) is a single long vector, in "natural" order
homPrf=reshape(profit(:,:,1),1,L*L); 

for j=1:L
for i=1:L
    if invt(i,j)>0
        homX(offZ + (j-1)*L+i) = -sqrt(invt(i,j));
continue
    end
        

    % conditional expectations of future value
    % conditional on our investment success
    W1=0; W0=0; W2=1;
    
    pr2=a*invt(j,i)/(1+a*invt(j,i));
    for di=0:1 % forgetting outcome
    for ii=0:1 % competitor investment outcome
        pr3 = d^di * (1-d)^(1-di) * pr2^ii * (1-pr2)^(1-ii);
        W1=W1 + pr3 * v( max(1,min(L,i+1-di)), max(1,min(L,j+ii-di)));
        W0=W0 + pr3 * v( max(1,min(L,i-di)), max(1,min(L,j+ii-di)));
        
        W2=W2-pr3; 
    end
    end
    if abs(W2)>1e-10
error('conditional expectation probability does not add up to 1')
    end
    
    homX(offZ + (j-1)*L+i) = sqrt(abs( a*B*(W1-W0) - (1+a*invt(i,j))^2 ));
    
end
end

if exist('hom_start.dat','file')
    delete('hom_start.dat'); % delete old file, since some systems cannot overwrite it
end
fid = fopen('hom_start.dat','w'); % open file for write
if fid==-1
    error('File open failed')
end
c1=fwrite(fid, [L B a a_end d d_end], 'float64');
        
c2=fwrite(fid,homX,'float64');
c3=fwrite(fid,homPrf,'float64');
c4=fclose(fid);
% check for success 
if (c1~=6) | ( c2~=length(homX) ) | ( c3~=length(homPrf) )
    error('params & X-vector writing failed')
end

if c4~=0
    error('file close failed')
end


%%% uncomment this line if you only want to generate starting value file
% disp('params & X-vector written to hom_start.dat; ready to run fortran file')
% return % stops execution of this script

%%% call homotopy %%%
%% Windows (make sure path and filename are correct)
eval('!QLD.exe') % 
% eval('!QLD.exe > batch.out') % output into a batch file
%% Unix (a.out is default filename of compiled fortran code)
% eval('!./a.out | batch.out') % "./" is necessary

copyfile('hom_start.dat','HomXpt/hom_start.dat')

% read output and compute Herfs
run QLD_ReadHom