	SUBROUTINE RHO(A,LAMBDA,X,H)

C	EVALUATE HOMOTOPY FUNCTION AT X, LAMBDA 
C	AND RETURN THE RESULT IN THE VECTOR H.
C
C	ECONOMIC MODEL: Quality ladder duopoly, no EE, GZ+Karl FOC
C
	USE mProfit ! profit function
C	declaration of inputs/outputs - do not change	
      USE REAL_PRECISION, ONLY : R8
      REAL (KIND=R8), INTENT(IN):: A(:),X(:)
      REAL (KIND=R8), INTENT(IN OUT):: LAMBDA
      REAL (KIND=R8), INTENT(OUT):: H(:)
C Reminders on coding and converting code from matlab to fortran
C * DECLARE ALL VARIABLES, fortran makes undeclared variables real*4, which is substantial loss of precision
C * IF has condition in (), requires THEN, closes with ENDIF
C * END means end of subroutine. loops close with something else  
C * Write integers as 2.0D0 when they are in same expression with reals 
C		(but make sure array indeces should integer of course)
C * Replace All ";" with "" - compiler is OK with ";", but not ADIFOR
C * Power is ** in Fotran

C	argument A is not 
!      INTEGER:: N
CCC
CCC Change everything below this - this is where you specify your model
CCC
C     COMMON model parameters
	integer L
	real*8 beta
	real*8 a_beg, a_end
	real*8 d_beg, d_end
	COMMON/ModelParams/beta,a_beg,a_end,d_beg,d_end,L
! offsets in X and H vectors - must be changed for new model
! not necessary, but make indexing and re-ordering columns/rows easier
	integer offV, offZ ! X vector: value V and GZ Zeta
	integer offBell, offFOC ! H vector: bellman and GZ FOC
	COMMON/OFFSETS/offV,offZ,offBell,offFOC

C	computed model parameters
	real*8 alpha, delta
C	service variables
	integer i,j,ii,di,ndx1
	real*8 W1, W0, prX, pr1, pr2, pr3
	real*8 invt1, invt2 ! our and rival's investment
	
C	recover model parameters from homotopy's lambda
	alpha = a_beg + LAMBDA * (a_end - a_beg)
	delta = d_beg + LAMBDA * (d_end - d_beg)

	DO 101 i=1,L
	DO 100 j=1,L
C		recover own and rival's investment
		ndx1 = ndx(i,j,L)
		invt1 = max(0.0,-X(offZ+ndx1))**2.0
		invt2 = max(0.0,-X(offZ+ndx(j,i,L)))**2.0

C    	conditional expectations of future value
C     conditional on our investment success
		W1=0.0D0 ! our investment works, cumulative sum
		W0=0.0D0 ! our investment fails, cumulative sum
!		prX=0.0D0 ! probability check, for debugging
    
		pr2 = alpha*invt2/(1.0D0+alpha*invt2)
		do 50 di=0,1 ! forgetting outcome
		do 50 ii=0,1 ! competitor investment outcome
			pr3=1.0D0

C			Original matlab code: pr3 = d^di * (1-d)^(1-di) * pr2^ii * (1-pr2)^(1-ii);
			if (di.eq.1) then
				pr3 = pr3*delta
			else
				pr3 = pr3*(1.0D0-delta)
			endif

			if (ii.eq.1) then
				pr3 = pr3*pr2
			else
				pr3 = pr3*(1.0D0-pr2)
			endif

			W1 = W1 + pr3 * X(offV + 
     &           ndx(max(1,min(L,i+1-di)), max(1,min(L,j+ii-di)), L ))
			W0 = W0 + pr3 * X(offV + 
     &           ndx(max(1,min(L,i-di)), max(1,min(L,j+ii-di)), L ))
C			Debug feature - print probabilities and values
!			IF ((i.eq.18).and.(j.eq.5)) then
!				write(*,'(I2,I2,F17.14,F20.14,F20.14)'), di, ii, pr3, 
!     &	X(offV+ndx(max(1,min(L,i+1-di)),max(1,min(L,j+ii-di)),L ))
!     &	,X(offV+ndx(max(1,min(L,i-di)), max(1,min(L,j+ii-di)), L ))
!			endif
			
        
!	        prX=prX+pr3; 
 50		continue

C	debug feature - check that probabilities add up to 1
!	    if (abs(prX-1.0D0)>1e-7) then
!			print*, 'CE prob minus 1.0 = ', prX
!		endif
    
C		Bellman equation
		pr1 = alpha*invt1/(1.0d0+alpha*invt1);
		H(offBell + ndx1) = -X(offV + ndx1) + Profit(ndx1) 
     &			-invt1 + beta * ( pr1*W1 + (1.0d0-pr1)*W0 )

C		FOC equation
		H(offFOC + ndx1) = alpha*beta*(W1-W0) 
     &		- (1.0d0+alpha*invt1)**2.0 + max(0.0,X(offZ+ndx1))**2.0

C	Debug feature - can put breakpoint on continue, so it stops at specific state
C	then use watch fields to evaluate expressions in your code and compare to matlab version of function
!		IF ((i.eq.18).and.(j.eq.5)) then
!			continue ! this does nothing, but you can put breakpoint on it
!		ENDIF

 100 	CONTINUE
 101 	CONTINUE

	END

	INTEGER FUNCTION ndx(i,j,L)
C	function ndx(i,j,L)
C	returns index of (i,j) in vectorized L*L matrix
C	probably should enforce (i,j) in [1:L], but it's ok in this short piece of code
	INTEGER i, j, L
		ndx = (j-1)*L+i
	END
