
close all

% printtitle=0;

[X,Y] = meshgrid(1:1:L);

% ============================================================================================================
% plot the equilibrium of the period game
if 0
	h=figure;
	set(h,'Position',[0 0 1025 690]) % use this position to plot utility, profit, and price functions
	
	% plot equilibrium price function
	subplot(2,2,2)
	mesh(X,Y,p(:,:,1));
	axis([1 L 1 L 0 1.1*max(max(p(:,:,1)))+1e-100])
	xlabel('\omega_2')
	ylabel('\omega_1')
	zlabel('price')
	title('Firm 1''s Equilibrium Price');
	
	% plot equilibrium profit function
	subplot(2,2,4)
	mesh(X,Y,profit(:,:,1));
	axis([1 L 1 L 0 1.1*max(max(profit(:,:,1)))+1e-100])
	% axis tight
	xlabel('\omega_2')
	ylabel('\omega_1')
	zlabel('profit');
	title('Firm 1''s Equilibrium Profit');
	
	% plot utility function
	subplot(2,2,1)
	plot(w,nu);
	axis ([1 L min(nu) 1.1*max(nu)]);
	xlabel('\omega_1');
	% ylabel('utility');
	% title('Consumers'' nu function');
	ylabel('valuation');
	title('Consumers'' Valuation');
	
	subplot(2,2,3)
	mesh(X,Y,D(:,:,1));
	axis([1 L 1 L 0 1.1*max(max(D(:,:,1)))+1e-100])
	xlabel('\omega_2')
	ylabel('\omega_1')
	zlabel('quantity demanded');
	title('Firm 1''s Equilibrium Quantity Demanded');
	
	if printtitle
        subtitle(sprintf('Product Market NE (L=%g, M=%g, c=%g, w^*=%g)\n',L,M,c,wstar));
	end
	
	set(gcf, 'PaperOrientation', 'landscape');
	set(gcf, 'PaperPositionMode', 'manual');
	set(gcf, 'PaperUnits', 'inches');
	set(gcf, 'PaperPosition', [.16 .65 10.68 7.19]);
end
% ============================================================================================================


% ============================================================================================================
% plot the MPE

h=figure;
% set(h,'Position',[0 0 1025 690]) % use this position to plot utility, profit, and price functions

% plot disv and disx1 for each iteration, and the stopping threshhold
subplot(2,2,2);
plot([1:1:length(disv)]',[log10(disv)' log10(disinvt)' log10(epsilon)*ones(1,length(disinvt))']);
% axis ([1 max(n,2) 0 1e-4]);

if min([epsilon,min(disv),min(disinvt)])~=0
    yaxismin=floor(log10(min([epsilon,min(disv),min(disinvt)])))-0.5;
else
    yaxismin=floor(log10(min([epsilon])))-0.5;
end

axis([1 max(numberitervfi,2) yaxismin (ceil(log10(max([epsilon,max(disv),max(disinvt)])))+0.1)]);
% axis ([1 max(n,2) (floor(log10(epsilon))-1/2) ceil(max(log10(distance)))]);
% axis tight;
xlabel('# of iterations');
% ylabel('');
title('log of relative distance between successive iterations');
legend('vf log_1_0(distance)','pf log_1_0(distance)','log_1_0(\epsilon)',0);

% plot firm 1's equilibrium value function
subplot(2,2,1);
mesh(X,Y,v1);
axis([1 L 1 L 0 1.1*max(max(v1(:,:,1)))+1e-100])
% axis tight;
% axis 'auto z';
xlabel('\omega_2')
ylabel('\omega_1')
zlabel('value')
% set(get(gca,'XLabel'),'Rotation',21.0)
% set(get(gca,'YLabel'),'Rotation',-29.0)
title('Firm 1''s Value Function')

% plot firm 1's equilibrium policy function
subplot(2,2,3);
mesh(X,Y,invt1);
axis([1 L 1 L 0 1.1*max(max(invt(:,:,1)))+1e-10])
% axis tight;
% axis 'auto z';
xlabel('\omega_2')
ylabel('\omega_1')
zlabel('investment')
% set(get(gca,'XLabel'),'Rotation',21.0)
% set(get(gca,'YLabel'),'Rotation',-29.0)
title('Firm 1''s Policy Function')

%%%% skip plotting lim/trans distr if there're n/a
limdisexists=exist('limdismatrix','var')
if ~exist('lasttransdis','var') & ~limdisexists
    return
end
    
    
if limdisexists
    subplot(2,2,4)
    mesh(X,Y,limdismatrix)
    zmax=max(max(limdismatrix))*1.1+1e-10;
    axis([1 L 1 L 0 zmax]);
    % axis tight;
    xlabel('\omega_2');
    ylabel('\omega_1');
    zlabel('prob(\omega)');
    title('Limiting Distribution');
else
    subplot(2,2,4)
    mesh(X,Y,lasttransdis)
    zmax=max(max(lasttransdis))*1.1+1e-10;
    axis([1 L 1 L 0 zmax]);
    % axis tight;
    xlabel('\omega_2');
    ylabel('\omega_1');
    zlabel('prob(\omega)');
    title(sprintf('Transient Distribution(t=%g)',lasttransdisnumber));
end

% if printtitle
subtitle(sprintf('MPE (L=%g, a=%g, d=%g, B=%g)\n',L,a,d,B));
% end

set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperPosition', [.16 .65 10.68 7.19]);
% ============================================================================================================

%     % check
%     for i=1:L+1
%         for j=1:L+1
%             pw2sum(i,j)=sum(pw2(i,j,:));
%             pw1sum(i,j)=sum(pw1(i,j,:));
%         end
%     end
    
% ============================================================================================================
% plot transient distributions, and limiting distribution (if it exists)
h=figure;
% set(h,'Position',[0 0 1025 690]) % use this position to plot utility, profit, and price functions

for i=1:8
    subplot(3,3,i)
    mesh(X,Y,squeeze(transgraph(i,:,:)))
    zmax=max(max(squeeze(transgraph(i,:,:))))*1.1+1e-10;
    axis([1 L 1 L 0 zmax]);
    % axis tight;
    xlabel('\omega_2');
    ylabel('\omega_1');
    zlabel('prob(\omega)');
    title(sprintf('t=%g',TT(i)));
end

if limdisexists
    subplot(3,3,9)
    mesh(X,Y,limdismatrix)
    zmax=max(max(limdismatrix))*1.1+1e-10;
    axis([1 L 1 L 0 zmax]);
    % axis tight;
    xlabel('\omega_2');
    ylabel('\omega_1');
    zlabel('prob(\omega)');
    title('Limiting Distribution');
else
    subplot(3,3,9)
    mesh(X,Y,lasttransdis)
    zmax=max(max(lasttransdis))*1.1+1e-10;
    axis([1 L 1 L 0 zmax]);
    % axis tight;
    xlabel('\omega_2');
    ylabel('\omega_1');
    zlabel('prob(\omega)');
    title(sprintf('t=%g',lasttransdisnumber));
end

if printtitle
    if limdisexists
        subtitle(sprintf('Transient and Limiting Distributions\n'));
    else
        subtitle(sprintf('Transient Distributions\n'));
    end
end

set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperPosition', [.16 .65 10.68 7.19]);
% ============================================================================================================


