clear all
close all

replace    = 0;    % set to 1 to replace equilibrium saved for the parameter values chosen, if it has been computed
save1      = 1;    % set to 1 to save MPE computed
printtitle = 1;    % set to 1 to print titles over figures (using 'subtitle.m')
maxtrans   = 1000; % maximum # of transient distributions
% global L M c a d B w g profit

%==================================================================================================================
%%% parameter values are set below
L = 18;    % # of states
% period game parameters
wstar = 12; % state at which utility function flattens out
M = 5;    % market size
c = 5;     % MC of production
% dynamic parameters
a = 15.0;     % effectiveness of investment (alpha)
d = 0.7;   % industry-wide depreciation (delta)
B = 0.925; % discount factor (beta)

% technical parameters
epsilon = 1e-12; % stopping threshold
maxiter = 1000; % maximum # of iterations in value function iteration algorithm
p0 = [10, 10]; % set the starting value for fsolve used to solve period game
st = [0 0]; % set the period 0 state to be used in computation of transient distributions, e.g., (w_1,w_2)=[0 0]
%=============================================================================================================================

w=[1:1:L]';

fileX = sprintf('equilibria/L=%d,M=%d,c=%d,a=%0.2g,d=%0.2g,,B=%0.3g,wstar=%g.mat',L,M,c,a,d,B,wstar); 

if replace==1
    if exist(fileX, 'file')
        delete(fileX)
        fprintf('\nOld equilibrium was deleted.\n')
    end
end


if exist(fileX, 'file')
    load(fileX)
    sprintf('File loaded from ''equilibria'' folder.')
else
    % compute profit function
    run QLDsetup
    
	v=profit(:,:,1)./(1-B); % set the initial value function
	invt=ones(L,L); % set the initial policy function

    if prodNEfound % proceed only if a NE for the product market has been found (or loaded)
        % compute equilibrum
        disp('Computing equilibrium in QLD game - PM algorithm')
        run Qldvfi
		disp(sprintf(' PM algorithm done, # iters = %d, time = %.2f seconds',n,t))
        
        % compute limiting distribution & plot
        % delete "0%" to enable it
        if 0%dis<=epsilon % 
            run QLDlimdis
            run QLDtransdis
        end        
        
        % save results in 'equilibria' folder only if value function iteration converges
        if dis<=epsilon & save1
            save(fileX);
        end    
    
    else
        fprintf('Product market NE has not been found: fsolve did not converge for at least one state.')
    end
end

if prodNEfound
    run QLDfigures
end