% == QUALITY LADDER DUOPOLY EXERCISE
% == By RON N. BORKOVSKY
% == FILENAME: QLDsetup.m
% == NOTE: This script solves for the nash equilbrium price matrices
% == in the product market game.  This script is run by the QLDvfi.m
% == script.

% global L M c a d B w g profit

% % parameter values are set below
% L = 18;
% M = 5;
% c = 5;
% a = 3;
% d = 0.7;
% B = 0.925;
% 
% % create the vector g (consumer utility for each state, excluding taste differences among consumers)
% w=[1:1:L]';
% g = 3*w-4;
% for i=6:L
%     g(i,:) = 12+log(2-exp(16-3*i));
% end
fileY = sprintf('PeriodProfit/L=%d,M=%d,c=%d,wstar=%g.mat',L,M,c,wstar); 

if exist(fileY, 'file')
    load(fileY);
else
    disp('solving period games')


% this is the PM utility function    
nu = w;
for i=wstar+1:L
    nu(i) = wstar + log(2 - exp(-(i-(wstar))));
end   

% % below is the consumers' utility function from Uli's assignment
% nu = 3*w-4;
% for i=6:L+1
%     nu(i,:) = 12+log(2-exp(16-3*i));
% end

% nu=1.5*w+2;
% aaa=9;
% for i=aaa:L+1
%     nu(i)=2+(aaa-1)*1.5;
% end

% p0 = [5, 5]; % set the starting value for fsolve, used in the loop below
% p=zeros(L,L,2); %create matrix for equilibrium prices
% profit=zeros(L,L,2); %create matrix for profits
n1 = 0;
n2 = 0;

prodNEfound=1;
% the loop below calculates equilibrium prices and profits for each state
for i=1:L
    n1=n1+1;
    for j=1:L
        n2=n2+1;
        [x,fval,exitflag] = fsolve(@myfun,p0,optimset(optimset('fsolve') ...
                       ,'Display','off'),[i,j],nu,c);
        p(i,j,1)=x(1); p(i,j,2)=x(2);
%         if exitflag < 1, fprintf('does not converge\n'); donotsave=1; break; end
        if exitflag < 1, fprintf('does not converge\n'); prodNEfound=0; end
        pp=myprof(x,[i,j],nu,c,M);
        profit(i,j,1)=pp(1); profit(i,j,2)=pp(2); 
        
        D(i,j) = M*exp(nu(i)-p(i,j,1))/(1+exp(nu(i)-p(i,j,1))+exp(nu(j)-p(i,j,2))); % compute firm 1's equilibrium quanity demanded
    end
end    

if prodNEfound
    save(fileY,'profit','p','D','nu','prodNEfound');
else
    disp('!!! period game solution failed !!!')
end



end